use crate::{server::Server, wire::Response, Id};
use async_std::{
    net::UdpSocket,
    sync::{Arc, Receiver, RwLock},
    task,
};
use std::{
    collections::BTreeMap,
    net::SocketAddr,
    sync::atomic::{AtomicBool, Ordering},
};

pub struct Endpoint {
    running: AtomicBool,
    socket: Arc<UdpSocket>,
    bind: String,
    clients: RwLock<BTreeMap<Id, Client>>,
}

pub struct Client {}

pub struct SendHandle {
    rx: Receiver<(SocketAddr, Response)>,
    socket: Arc<UdpSocket>,
}

impl SendHandle {
    async fn run(self) {
        // Loop until 
        while let Some((peer, resp)) = self.rx.recv().await {
            
        }
    }
}

impl Endpoint {
    pub async fn new(bind: &str) -> Arc<Self> {
        let socket = UdpSocket::bind(bind).await.unwrap();
        Arc::new(Self {
            running: true.into(),
            socket: Arc::new(socket),
            bind: bind.into(),
            clients: Default::default(),
        })
    }

    /// Stop the endpoint
    pub fn stop(self: &Arc<Self>) {
        self.running.store(false, Ordering::Relaxed);
    }

    /// Start listening for incoming packets
    pub async fn listen(self: &Arc<Self>, serv: Arc<Server>) {
        let mut buf = vec![0; 1024];

        info!("Listening for connections on {}", self.bind);
        while self.running.load(Ordering::Relaxed) {
            let (int, peer) = self.socket.recv_from(&mut buf).await.unwrap();
            if int > 1024 {
                warn!("Read a larger chunk than buffer?");
            }
        }
    }
}
